@@ -15,7 +15,6 @@
 from functools import wraps
 
 from neutron import context as ncontext
-from oslo_log import log as logging
 from oslo_utils import excutils
 
 from neutron_lbaas.common import exceptions
@@ -24,9 +23,6 @@ from neutron_lbaas.drivers import driver_mixins
 from neutron_lbaas.services.loadbalancer import constants
 
 
-LOG = logging.getLogger(__name__)
-
-
 class NotImplementedManager(object):
     """Helper class to make any subclass of LoadBalancerBaseDriver explode if
     it is missing any of the required object managers.
@@ -87,6 +83,23 @@ class BaseLoadBalancerManager(driver_mixins.BaseRefreshMixin,
                               driver_mixins.BaseManagerMixin):
     model_class = models.LoadBalancer
 
+    @property
+    def allows_create_graph(self):
+        """
+        Can this driver create a load balancer graph in one call.
+
+        Return True if this driver has the capability to create a load balancer
+        and any of its children in one driver call.  If this returns True and
+        the user requests the creation of a load balancer graph, then the
+        create_graph method will be called to create the load balancer.
+        """
+        return False
+
+    @property
+    def allows_healthmonitor_thresholds(self):
+        """Does this driver support thresholds for health monitors"""
+        return False
+
     @property
     def allocates_vip(self):
         """Does this driver need to allocate its own virtual IPs"""
@@ -163,10 +176,12 @@ def driver_op(func):
     @wraps(func)
     def func_wrapper(*args, **kwargs):
         d = (func.__name__ == 'delete')
+        lb_create = ((func.__name__ == 'create') and
+                     isinstance(args[0], BaseLoadBalancerManager))
         try:
             r = func(*args, **kwargs)
             args[0].successful_completion(
-                args[1], args[2], delete=d)
+                args[1], args[2], delete=d, lb_create=lb_create)
             return r
         except Exception:
             with excutils.save_and_reraise_exception():
