@@ -13,9 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import six
 
 from neutron.api.v2 import attributes as attrs
+from neutron.api.v2 import base as napi_base
 from neutron import context as ncontext
 from neutron.db import servicetype_db as st_db
 from neutron.extensions import flavors
@@ -24,357 +26,34 @@ from neutron.plugins.common import constants
 from neutron.services.flavors import flavors_plugin
 from neutron.services import provider_configuration as pconf
 from neutron.services import service_base
+from neutron_lib import constants as n_constants
 from neutron_lib import exceptions as n_exc
-from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import excutils
+from oslo_utils import encodeutils
 
 from neutron_lbaas._i18n import _LI, _LE
 from neutron_lbaas import agent_scheduler as agent_scheduler_v2
 import neutron_lbaas.common.cert_manager
 from neutron_lbaas.common.tls_utils import cert_parser
-from neutron_lbaas.db.loadbalancer import loadbalancer_db as ldb
 from neutron_lbaas.db.loadbalancer import loadbalancer_dbv2 as ldbv2
 from neutron_lbaas.db.loadbalancer import models
+from neutron_lbaas.extensions import l7
+from neutron_lbaas.extensions import lb_graph as lb_graph_ext
 from neutron_lbaas.extensions import lbaas_agentschedulerv2
-from neutron_lbaas.extensions import loadbalancer as lb_ext
 from neutron_lbaas.extensions import loadbalancerv2
 from neutron_lbaas.extensions import sharedpools
-from neutron_lbaas.services.loadbalancer import agent_scheduler
 from neutron_lbaas.services.loadbalancer import constants as lb_const
 from neutron_lbaas.services.loadbalancer import data_models
 LOG = logging.getLogger(__name__)
 CERT_MANAGER_PLUGIN = neutron_lbaas.common.cert_manager.get_backend()
 
 
-def verify_lbaas_mutual_exclusion():
-    """Verifies lbaas v1 and lbaas v2 cannot be active concurrently."""
-    plugins = set([LoadBalancerPlugin.__name__, LoadBalancerPluginv2.__name__])
-    cfg_sps = set([sp.split('.')[-1] for sp in cfg.CONF.service_plugins])
-
-    if len(plugins.intersection(cfg_sps)) >= 2:
-        msg = _LE("Cannot have service plugins %(v1)s and %(v2)s active at "
-                  "the same time!") % {'v1': LoadBalancerPlugin.__name__,
-                                       'v2': LoadBalancerPluginv2.__name__}
-        LOG.error(msg)
-        raise SystemExit(1)
-
-
 def add_provider_configuration(type_manager, service_type):
     type_manager.add_provider_configuration(
         service_type,
         pconf.ProviderConfiguration('neutron_lbaas'))
 
 
-class LoadBalancerPlugin(ldb.LoadBalancerPluginDb,
-                         agent_scheduler.LbaasAgentSchedulerDbMixin):
-    """Implementation of the Neutron Loadbalancer Service Plugin.
-
-    This class manages the workflow of LBaaS request/response.
-    Most DB related works are implemented in class
-    loadbalancer_db.LoadBalancerPluginDb.
-    """
-    supported_extension_aliases = ["lbaas",
-                                   "lbaas_agent_scheduler",
-                                   "service-type"]
-    path_prefix = lb_ext.LOADBALANCER_PREFIX
-
-    # lbaas agent notifiers to handle agent update operations;
-    # can be updated by plugin drivers while loading;
-    # will be extracted by neutron manager when loading service plugins;
-    agent_notifiers = {}
-
-    def __init__(self):
-        """Initialization for the loadbalancer service plugin."""
-        self.service_type_manager = st_db.ServiceTypeManager.get_instance()
-        add_provider_configuration(
-            self.service_type_manager, constants.LOADBALANCER)
-        self._load_drivers()
-        super(LoadBalancerPlugin, self).subscribe()
-
-    def _load_drivers(self):
-        """Loads plugin-drivers specified in configuration."""
-        self.drivers, self.default_provider = service_base.load_drivers(
-            constants.LOADBALANCER, self)
-
-        # NOTE(blogan): this method MUST be called after
-        # service_base.load_drivers to correctly verify
-        verify_lbaas_mutual_exclusion()
-
-        ctx = ncontext.get_admin_context()
-        # stop service in case provider was removed, but resources were not
-        self._check_orphan_pool_associations(ctx, self.drivers.keys())
-
-    def _check_orphan_pool_associations(self, context, provider_names):
-        """Checks remaining associations between pools and providers.
-
-        If admin has not undeployed resources with provider that was deleted
-        from configuration, neutron service is stopped. Admin must delete
-        resources prior to removing providers from configuration.
-        """
-        pools = self.get_pools(context)
-        lost_providers = set([pool['provider'] for pool in pools
-                              if pool['provider'] not in provider_names])
-        # resources are left without provider - stop the service
-        if lost_providers:
-            LOG.exception(_LE("Delete associated loadbalancer pools before "
-                              "removing providers %s"),
-                          list(lost_providers))
-            raise SystemExit(1)
-
-    def _get_driver_for_provider(self, provider):
-        if provider in self.drivers:
-            return self.drivers[provider]
-        # raise if not associated (should never be reached)
-        raise n_exc.Invalid(_LE("Error retrieving driver for provider %s") %
-                            provider)
-
-    def _get_driver_for_pool(self, context, pool_id):
-        pool = self.get_pool(context, pool_id)
-        try:
-            return self.drivers[pool['provider']]
-        except KeyError:
-            raise n_exc.Invalid(_LE("Error retrieving provider for pool %s") %
-                                pool_id)
-
-    def get_plugin_type(self):
-        return constants.LOADBALANCER
-
-    def get_plugin_description(self):
-        return "Neutron LoadBalancer Service Plugin"
-
-    def create_vip(self, context, vip):
-        v = super(LoadBalancerPlugin, self).create_vip(context, vip)
-        driver = self._get_driver_for_pool(context, v['pool_id'])
-        driver.create_vip(context, v)
-        return v
-
-    def update_vip(self, context, id, vip):
-        if 'status' not in vip['vip']:
-            vip['vip']['status'] = constants.PENDING_UPDATE
-        old_vip = self.get_vip(context, id)
-        v = super(LoadBalancerPlugin, self).update_vip(context, id, vip)
-        driver = self._get_driver_for_pool(context, v['pool_id'])
-        driver.update_vip(context, old_vip, v)
-        return v
-
-    def _delete_db_vip(self, context, id):
-        # proxy the call until plugin inherits from DBPlugin
-        super(LoadBalancerPlugin, self).delete_vip(context, id)
-
-    def delete_vip(self, context, id):
-        self.update_status(context, ldb.Vip,
-                           id, constants.PENDING_DELETE)
-        v = self.get_vip(context, id)
-        driver = self._get_driver_for_pool(context, v['pool_id'])
-        driver.delete_vip(context, v)
-
-    def _get_provider_name(self, context, pool):
-        if ('provider' in pool and
-            pool['provider'] != attrs.ATTR_NOT_SPECIFIED):
-            provider_name = pconf.normalize_provider_name(pool['provider'])
-            self.validate_provider(provider_name)
-            return provider_name
-        else:
-            if not self.default_provider:
-                raise pconf.DefaultServiceProviderNotFound(
-                    service_type=constants.LOADBALANCER)
-            return self.default_provider
-
-    def create_pool(self, context, pool):
-        # This validation is because the new API version also has a resource
-        # called pool and these attributes have to be optional in the old API
-        # so they are not required attributes of the new.  Its complicated.
-        if pool['pool']['lb_method'] == attrs.ATTR_NOT_SPECIFIED:
-            raise loadbalancerv2.RequiredAttributeNotSpecified(
-                attr_name='lb_method')
-        if pool['pool']['subnet_id'] == attrs.ATTR_NOT_SPECIFIED:
-            raise loadbalancerv2.RequiredAttributeNotSpecified(
-                attr_name='subnet_id')
-
-        provider_name = self._get_provider_name(context, pool['pool'])
-        p = super(LoadBalancerPlugin, self).create_pool(context, pool)
-
-        self.service_type_manager.add_resource_association(
-            context,
-            constants.LOADBALANCER,
-            provider_name, p['id'])
-        # need to add provider name to pool dict,
-        # because provider was not known to db plugin at pool creation
-        p['provider'] = provider_name
-        driver = self.drivers[provider_name]
-        try:
-            driver.create_pool(context, p)
-        except lb_ext.NoEligibleBackend:
-            # that should catch cases when backend of any kind
-            # is not available (agent, appliance, etc)
-            self.update_status(context, ldb.Pool,
-                               p['id'], constants.ERROR,
-                               "No eligible backend")
-            raise lb_ext.NoEligibleBackend(pool_id=p['id'])
-        return p
-
-    def update_pool(self, context, id, pool):
-        if 'status' not in pool['pool']:
-            pool['pool']['status'] = constants.PENDING_UPDATE
-        old_pool = self.get_pool(context, id)
-        p = super(LoadBalancerPlugin, self).update_pool(context, id, pool)
-        driver = self._get_driver_for_provider(p['provider'])
-        driver.update_pool(context, old_pool, p)
-        return p
-
-    def _delete_db_pool(self, context, id):
-        # proxy the call until plugin inherits from DBPlugin
-        # rely on uuid uniqueness:
-        try:
-            with context.session.begin(subtransactions=True):
-                self.service_type_manager.del_resource_associations(
-                    context, [id])
-                super(LoadBalancerPlugin, self).delete_pool(context, id)
-        except Exception:
-            # that should not happen
-            # if it's still a case - something goes wrong
-            # log the error and mark the pool as ERROR
-            LOG.error(_LE('Failed to delete pool %s, putting it in ERROR '
-                          'state'),
-                      id)
-            with excutils.save_and_reraise_exception():
-                self.update_status(context, ldb.Pool,
-                                   id, constants.ERROR)
-
-    def delete_pool(self, context, id):
-        # check for delete conditions and update the status
-        # within a transaction to avoid a race
-        with context.session.begin(subtransactions=True):
-            self.update_status(context, ldb.Pool,
-                               id, constants.PENDING_DELETE)
-            self._ensure_pool_delete_conditions(context, id)
-        p = self.get_pool(context, id)
-        driver = self._get_driver_for_provider(p['provider'])
-        driver.delete_pool(context, p)
-
-    def create_member(self, context, member):
-        m = super(LoadBalancerPlugin, self).create_member(context, member)
-        driver = self._get_driver_for_pool(context, m['pool_id'])
-        driver.create_member(context, m)
-        return m
-
-    def update_member(self, context, id, member):
-        if 'status' not in member['member']:
-            member['member']['status'] = constants.PENDING_UPDATE
-        old_member = self.get_member(context, id)
-        m = super(LoadBalancerPlugin, self).update_member(context, id, member)
-        driver = self._get_driver_for_pool(context, m['pool_id'])
-        driver.update_member(context, old_member, m)
-        return m
-
-    def _delete_db_member(self, context, id):
-        # proxy the call until plugin inherits from DBPlugin
-        super(LoadBalancerPlugin, self).delete_member(context, id)
-
-    def delete_member(self, context, id):
-        self.update_status(context, ldb.Member,
-                           id, constants.PENDING_DELETE)
-        m = self.get_member(context, id)
-        driver = self._get_driver_for_pool(context, m['pool_id'])
-        driver.delete_member(context, m)
-
-    def _validate_hm_parameters(self, delay, timeout):
-        if delay < timeout:
-            raise lb_ext.DelayOrTimeoutInvalid()
-
-    def create_health_monitor(self, context, health_monitor):
-        new_hm = health_monitor['health_monitor']
-        self._validate_hm_parameters(new_hm['delay'], new_hm['timeout'])
-
-        hm = super(LoadBalancerPlugin, self).create_health_monitor(
-            context,
-            health_monitor
-        )
-        return hm
-
-    def update_health_monitor(self, context, id, health_monitor):
-        new_hm = health_monitor['health_monitor']
-        old_hm = self.get_health_monitor(context, id)
-        delay = new_hm.get('delay', old_hm.get('delay'))
-        timeout = new_hm.get('timeout', old_hm.get('timeout'))
-        self._validate_hm_parameters(delay, timeout)
-
-        hm = super(LoadBalancerPlugin, self).update_health_monitor(
-            context,
-            id,
-            health_monitor
-        )
-
-        with context.session.begin(subtransactions=True):
-            qry = context.session.query(
-                ldb.PoolMonitorAssociation
-            ).filter_by(monitor_id=hm['id']).join(ldb.Pool)
-            for assoc in qry:
-                driver = self._get_driver_for_pool(context, assoc['pool_id'])
-                driver.update_pool_health_monitor(context, old_hm,
-                                                  hm, assoc['pool_id'])
-        return hm
-
-    def _delete_db_pool_health_monitor(self, context, hm_id, pool_id):
-        super(LoadBalancerPlugin, self).delete_pool_health_monitor(context,
-                                                                   hm_id,
-                                                                   pool_id)
-
-    def _delete_db_health_monitor(self, context, id):
-        super(LoadBalancerPlugin, self).delete_health_monitor(context, id)
-
-    def create_pool_health_monitor(self, context, health_monitor, pool_id):
-        retval = super(LoadBalancerPlugin, self).create_pool_health_monitor(
-            context,
-            health_monitor,
-            pool_id
-        )
-        monitor_id = health_monitor['health_monitor']['id']
-        hm = self.get_health_monitor(context, monitor_id)
-        driver = self._get_driver_for_pool(context, pool_id)
-        driver.create_pool_health_monitor(context, hm, pool_id)
-        return retval
-
-    def delete_pool_health_monitor(self, context, id, pool_id):
-        self.update_pool_health_monitor(context, id, pool_id,
-                                        constants.PENDING_DELETE)
-        hm = self.get_health_monitor(context, id)
-        driver = self._get_driver_for_pool(context, pool_id)
-        driver.delete_pool_health_monitor(context, hm, pool_id)
-
-    def stats(self, context, pool_id):
-        driver = self._get_driver_for_pool(context, pool_id)
-        stats_data = driver.stats(context, pool_id)
-        # if we get something from the driver -
-        # update the db and return the value from db
-        # else - return what we have in db
-        if stats_data:
-            super(LoadBalancerPlugin, self).update_pool_stats(
-                context,
-                pool_id,
-                stats_data
-            )
-        return super(LoadBalancerPlugin, self).stats(context,
-                                                     pool_id)
-
-    def populate_vip_graph(self, context, vip):
-        """Populate the vip with: pool, members, healthmonitors."""
-
-        pool = self.get_pool(context, vip['pool_id'])
-        vip['pool'] = pool
-        vip['members'] = [self.get_member(context, member_id)
-                          for member_id in pool['members']]
-        vip['health_monitors'] = [self.get_health_monitor(context, hm_id)
-                                  for hm_id in pool['health_monitors']]
-        return vip
-
-    def validate_provider(self, provider):
-        if provider not in self.drivers:
-            raise pconf.ServiceProviderNotFound(
-                provider=provider, service_type=constants.LOADBALANCER)
-
-
 class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
     """Implementation of the Neutron Loadbalancer Service Plugin.
 
@@ -386,7 +65,10 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
                                    "shared_pools",
                                    "l7",
                                    "lbaas_agent_schedulerv2",
-                                   "service-type"]
+                                   "service-type",
+                                   "lb-graph",
+                                   "lb_network_vip",
+                                   "hm_max_retries_down"]
     path_prefix = loadbalancerv2.LOADBALANCERV2_PREFIX
 
     agent_notifiers = (
@@ -415,10 +97,6 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         self.drivers, self.default_provider = service_base.load_drivers(
             constants.LOADBALANCERV2, self)
 
-        # NOTE(blogan): this method MUST be called after
-        # service_base.load_drivers to correctly verify
-        verify_lbaas_mutual_exclusion()
-
         ctx = ncontext.get_admin_context()
         # stop service in case provider was removed, but resources were not
         self._check_orphan_loadbalancer_associations(ctx, self.drivers.keys())
@@ -462,7 +140,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
 
     def _get_provider_name(self, entity):
         if ('provider' in entity and
-                entity['provider'] != attrs.ATTR_NOT_SPECIFIED):
+                entity['provider'] != n_constants.ATTR_NOT_SPECIFIED):
             provider_name = pconf.normalize_provider_name(entity['provider'])
             del entity['provider']
             self.validate_provider(provider_name)
@@ -470,28 +148,29 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         else:
             if not self.default_provider:
                 raise pconf.DefaultServiceProviderNotFound(
-                    service_type=constants.LOADBALANCER)
-            del entity['provider']
+                    service_type=constants.LOADBALANCERV2)
+            if entity.get('provider'):
+                del entity['provider']
             return self.default_provider
 
     def _call_driver_operation(self, context, driver_method, db_entity,
-                               old_db_entity=None):
+                               old_db_entity=None, **kwargs):
         manager_method = "%s.%s" % (driver_method.__self__.__class__.__name__,
                                     driver_method.__name__)
         LOG.info(_LI("Calling driver operation %s") % manager_method)
         try:
             if old_db_entity:
-                driver_method(context, old_db_entity, db_entity)
+                driver_method(context, old_db_entity, db_entity, **kwargs)
             else:
-                driver_method(context, db_entity)
+                driver_method(context, db_entity, **kwargs)
         # catching and reraising agent issues
         except (lbaas_agentschedulerv2.NoEligibleLbaasAgent,
                 lbaas_agentschedulerv2.NoActiveLbaasAgent) as no_agent:
             raise no_agent
-        except Exception:
+        except Exception as e:
             LOG.exception(_LE("There was an error in the driver"))
             self._handle_driver_error(context, db_entity)
-            raise loadbalancerv2.DriverError()
+            raise loadbalancerv2.DriverError(msg=e)
 
     def _handle_driver_error(self, context, db_entity):
         lb_id = db_entity.root_loadbalancer.id
@@ -530,7 +209,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         # the provider, when flavor templating arrives.
 
         if ('provider' in loadbalancer and
-            loadbalancer['provider'] != attrs.ATTR_NOT_SPECIFIED):
+            loadbalancer['provider'] != n_constants.ATTR_NOT_SPECIFIED):
             raise loadbalancerv2.ProviderFlavorConflict()
 
         plugin = manager.NeutronManager.get_service_plugins().get(
@@ -562,9 +241,117 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
 
         loadbalancer['provider'] = provider
 
+    def _get_tweaked_resource_attribute_map(self):
+        memo = {id(n_constants.ATTR_NOT_SPECIFIED):
+                n_constants.ATTR_NOT_SPECIFIED}
+        ram = copy.deepcopy(attrs.RESOURCE_ATTRIBUTE_MAP, memo=memo)
+        del ram['listeners']['loadbalancer_id']
+        del ram['pools']['listener_id']
+        del ram['healthmonitors']['pool_id']
+        for resource in ram:
+            if resource in lb_graph_ext.EXISTING_ATTR_GRAPH_ATTR_MAP:
+                ram[resource].update(
+                    lb_graph_ext.EXISTING_ATTR_GRAPH_ATTR_MAP[resource])
+        return ram
+
+    def _prepare_loadbalancer_graph(self, context, loadbalancer):
+        """Prepares the entire user requested body of a load balancer graph
+
+        To minimize code duplication, this method reuses the neutron API
+        controller's method to do all the validation, conversion, and
+        defaulting of each resource.  This reuses the RESOURCE_ATTRIBUTE_MAP
+        and SUB_RESOURCE_ATTRIBUTE_MAP from the extension to enable this.
+        """
+        # NOTE(blogan): it is assumed the loadbalancer attributes have already
+        # passed through the prepare_request_body method by nature of the
+        # normal neutron wsgi workflow.  So we start with listeners since
+        # that probably has not passed through the neutron wsgi workflow.
+        ram = self._get_tweaked_resource_attribute_map()
+        # NOTE(blogan): members are not populated in the attributes.RAM so
+        # our only option is to use the original extension definition of member
+        # to validate.  If members ever need something added to it then it too
+        # will need to be added here.
+        prepped_lb = napi_base.Controller.prepare_request_body(
+            context, {'loadbalancer': loadbalancer}, True, 'loadbalancer',
+            ram['loadbalancers']
+        )
+        sub_ram = loadbalancerv2.SUB_RESOURCE_ATTRIBUTE_MAP
+        sub_ram.update(l7.SUB_RESOURCE_ATTRIBUTE_MAP)
+        prepped_listeners = []
+        for listener in loadbalancer.get('listeners', []):
+            prepped_listener = napi_base.Controller.prepare_request_body(
+                context, {'listener': listener}, True, 'listener',
+                ram['listeners'])
+            l7policies = listener.get('l7policies')
+            if l7policies and l7policies != n_constants.ATTR_NOT_SPECIFIED:
+                prepped_policies = []
+                for policy in l7policies:
+                    prepped_policy = napi_base.Controller.prepare_request_body(
+                        context, {'l7policy': policy}, True, 'l7policy',
+                        ram['l7policies'])
+                    l7rules = policy.get('rules')
+                    redirect_pool = policy.get('redirect_pool')
+                    if l7rules and l7rules != n_constants.ATTR_NOT_SPECIFIED:
+                        prepped_rules = []
+                        for rule in l7rules:
+                            prepped_rule = (
+                                napi_base.Controller.prepare_request_body(
+                                    context, {'l7rule': rule}, True, 'l7rule',
+                                    sub_ram['rules']['parameters']))
+                            prepped_rules.append(prepped_rule)
+                        prepped_policy['l7_rules'] = prepped_rules
+                    if (redirect_pool and
+                            redirect_pool != n_constants.ATTR_NOT_SPECIFIED):
+                        prepped_r_pool = (
+                            napi_base.Controller.prepare_request_body(
+                                context, {'pool': redirect_pool}, True, 'pool',
+                                ram['pools']))
+                        prepped_r_members = []
+                        for member in redirect_pool.get('members', []):
+                            prepped_r_member = (
+                                napi_base.Controller.prepare_request_body(
+                                    context, {'member': member},
+                                    True, 'member',
+                                    sub_ram['members']['parameters']))
+                            prepped_r_members.append(prepped_r_member)
+                        prepped_r_pool['members'] = prepped_r_members
+                        r_hm = redirect_pool.get('healthmonitor')
+                        if r_hm and r_hm != n_constants.ATTR_NOT_SPECIFIED:
+                            prepped_r_hm = (
+                                napi_base.Controller.prepare_request_body(
+                                    context, {'healthmonitor': r_hm},
+                                    True, 'healthmonitor',
+                                    ram['healthmonitors']))
+                            prepped_r_pool['healthmonitor'] = prepped_r_hm
+                        prepped_policy['redirect_pool'] = redirect_pool
+                    prepped_policies.append(prepped_policy)
+                prepped_listener['l7_policies'] = prepped_policies
+            pool = listener.get('default_pool')
+            if pool and pool != n_constants.ATTR_NOT_SPECIFIED:
+                prepped_pool = napi_base.Controller.prepare_request_body(
+                    context, {'pool': pool}, True, 'pool',
+                    ram['pools'])
+                prepped_members = []
+                for member in pool.get('members', []):
+                    prepped_member = napi_base.Controller.prepare_request_body(
+                        context, {'member': member}, True, 'member',
+                        sub_ram['members']['parameters'])
+                    prepped_members.append(prepped_member)
+                prepped_pool['members'] = prepped_members
+                hm = pool.get('healthmonitor')
+                if hm and hm != n_constants.ATTR_NOT_SPECIFIED:
+                    prepped_hm = napi_base.Controller.prepare_request_body(
+                        context, {'healthmonitor': hm}, True, 'healthmonitor',
+                        ram['healthmonitors'])
+                    prepped_pool['healthmonitor'] = prepped_hm
+                prepped_listener['default_pool'] = prepped_pool
+            prepped_listeners.append(prepped_listener)
+        prepped_lb['listeners'] = prepped_listeners
+        return loadbalancer
+
     def create_loadbalancer(self, context, loadbalancer):
         loadbalancer = loadbalancer.get('loadbalancer')
-        if loadbalancer['flavor_id'] != attrs.ATTR_NOT_SPECIFIED:
+        if loadbalancer['flavor_id'] != n_constants.ATTR_NOT_SPECIFIED:
             self._insert_provider_name_from_flavor(context, loadbalancer)
         else:
             del loadbalancer['flavor_id']
@@ -583,6 +370,30 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         self._call_driver_operation(context, create_method, lb_db)
         return self.db.get_loadbalancer(context, lb_db.id).to_api_dict()
 
+    def create_graph(self, context, graph):
+        loadbalancer = graph.get('graph', {}).get('loadbalancer')
+        loadbalancer = self._prepare_loadbalancer_graph(context, loadbalancer)
+        if loadbalancer['flavor_id'] != n_constants.ATTR_NOT_SPECIFIED:
+            self._insert_provider_name_from_flavor(context, loadbalancer)
+        else:
+            del loadbalancer['flavor_id']
+        provider_name = self._get_provider_name(loadbalancer)
+        driver = self.drivers[provider_name]
+        if not driver.load_balancer.allows_create_graph:
+            raise lb_graph_ext.ProviderCannotCreateLoadBalancerGraph
+        lb_db = self.db.create_loadbalancer_graph(
+            context, loadbalancer,
+            allocate_vip=not driver.load_balancer.allocates_vip)
+        self.service_type_manager.add_resource_association(
+            context, constants.LOADBALANCERV2, provider_name, lb_db.id)
+        create_method = (driver.load_balancer.create_and_allocate_vip
+                         if driver.load_balancer.allocates_vip
+                         else driver.load_balancer.create)
+        self._call_driver_operation(context, create_method, lb_db)
+        api_lb = {'loadbalancer': self.db.get_loadbalancer(
+            context, lb_db.id).to_api_dict(full_graph=True)}
+        return api_lb
+
     def update_loadbalancer(self, context, id, loadbalancer):
         loadbalancer = loadbalancer.get('loadbalancer')
         old_lb = self.db.get_loadbalancer(context, id)
@@ -649,8 +460,9 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
                         container_id=container_ref)
                 else:
                     # Could be a keystone configuration error...
+                    err_msg = encodeutils.exception_to_unicode(e)
                     raise loadbalancerv2.CertManagerError(
-                        ref=container_ref, reason=e.message
+                        ref=container_ref, reason=err_msg
                     )
 
             try:
@@ -673,9 +485,9 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
                 validate_tls_container(container_ref)
 
         to_validate = []
-        if not listener['default_tls_container_ref']:
-            raise loadbalancerv2.TLSDefaultContainerNotSpecified()
         if not curr_listener:
+            if not listener['default_tls_container_ref']:
+                raise loadbalancerv2.TLSDefaultContainerNotSpecified()
             to_validate.extend([listener['default_tls_container_ref']])
             if 'sni_container_refs' in listener:
                 to_validate.extend(listener['sni_container_refs'])
@@ -717,11 +529,11 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
                 default_pool = self.db.get_pool(context, default_pool_id)
                 lb_id = default_pool.loadbalancer.id
                 listener['loadbalancer_id'] = lb_id
+            else:
+                self._check_pool_loadbalancer_match(
+                    context, default_pool_id, lb_id)
         elif not lb_id:
             raise sharedpools.ListenerMustHaveLoadbalancer()
-        if default_pool_id and lb_id:
-            self._check_pool_loadbalancer_match(
-                context, default_pool_id, lb_id)
         self.db.test_and_set_status(context, models.LoadBalancer, lb_id,
                                     constants.PENDING_UPDATE)
 
@@ -758,9 +570,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         try:
             curr_listener = curr_listener_db.to_dict()
 
-            default_tls_container_ref = listener.get(
-                'default_tls_container_ref')
-            if not default_tls_container_ref:
+            if 'default_tls_container_ref' not in listener:
                 listener['default_tls_container_ref'] = (
                     # NOTE(blogan): not changing to ref bc this dictionary is
                     # created from a data model
@@ -806,9 +616,9 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         old_listener = self.db.get_listener(context, id)
         if old_listener.l7_policies:
             raise loadbalancerv2.EntityInUse(
-                entity_using=models.L7Policy.NAME,
+                entity_using=models.Listener.NAME,
                 id=old_listener.l7_policies[0].id,
-                entity_in_use=models.Listener.NAME)
+                entity_in_use=models.L7Policy.NAME)
         self.db.test_and_set_status(context, models.Listener, id,
                                     constants.PENDING_DELETE)
         listener_db = self.db.get_listener(context, id)
@@ -899,11 +709,11 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
         return self.db.get_pool(context, id).to_api_dict()
 
     def delete_pool(self, context, id):
-        old_lb = self.db.get_pool(context, id)
-        if old_lb.healthmonitor:
+        old_pool = self.db.get_pool(context, id)
+        if old_pool.healthmonitor:
             raise loadbalancerv2.EntityInUse(
                 entity_using=models.HealthMonitorV2.NAME,
-                id=old_lb.healthmonitor.id,
+                id=old_pool.healthmonitor.id,
                 entity_in_use=models.PoolV2.NAME)
         self.db.test_and_set_status(context, models.PoolV2, id,
                                     constants.PENDING_DELETE)
@@ -926,12 +736,12 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
                                                 id=pool_id)
 
     def create_pool_member(self, context, pool_id, member):
-        self._check_pool_exists(context, pool_id)
+        member = member.get('member')
+        self.db.check_subnet_exists(context, member['subnet_id'])
         db_pool = self.db.get_pool(context, pool_id)
         self.db.test_and_set_status(context, models.LoadBalancer,
                                     db_pool.root_loadbalancer.id,
                                     constants.PENDING_UPDATE)
-        member = member.get('member')
         try:
             member_db = self.db.create_pool_member(context, member, pool_id)
         except Exception as exc:
@@ -1341,7 +1151,7 @@ class LoadBalancerPluginv2(loadbalancerv2.LoadBalancerPluginBaseV2):
                 [ps["id"] for ps in lb_status["pools"]]):
                 lb_status["pools"].append(pool_status)
             if self._is_degraded(curr_listener.default_pool):
-                self._set_degraded(self, listener_status, lb_status)
+                self._set_degraded(listener_status, lb_status)
             members = curr_listener.default_pool.members
             for curr_member in members:
                 if not curr_member.admin_state_up:
